unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, iDragDrop, Menus, IniFiles;

type
  TForm1 = class(TForm)
    GrpPack: TGroupBox;
    LblLevel: TLabel;
    TrkLevel: TTrackBar;
    GrpUnPack: TGroupBox;
    Label1: TLabel;
    MainMenu: TMainMenu;
    MenuUpx: TMenuItem;
    MenuPath: TMenuItem;
    MenuSays: TMenuItem;
    MenuLicense: TMenuItem;
    MenuVersion: TMenuItem;
    MenuPack: TMenuItem;
    MenuBackup: TMenuItem;
    MenuExport: TMenuItem;
    MenuIcon: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TrkLevelChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure MenuPathClick(Sender: TObject);
    procedure MenuSaysClick(Sender: TObject);
    procedure MenuLicenseClick(Sender: TObject);
    procedure MenuVersionClick(Sender: TObject);
  private
    FIniName    : string;
    FIniFile    : TIniFile;
    Dropper     : TShellDrop;
    FLock       : Boolean;

    FStdOut, FErrOut: TStringList;
    procedure   ClearOut();
    procedure   ShowOut();

    procedure   DroppedAction(AFiles, AFolds: TStringList;
                            AControl: TControl; APos: TPoint);
    procedure   DoLevelCompress(const Path: string);
    procedure   DoBestCompress(const Path: string);
    procedure   DoDeCompress(const Path: string);

    function    UpxPath(): string;
    procedure   SetEnable();
    procedure   ExecuteWithShow(const Line: string);

    procedure   LoadSetting();
    procedure   SaveSetting();
  public

  end;

var
  Form1: TForm1;

implementation

uses
    GrabConsole, fjutil;

{$R *.dfm}

//------------------------------------------------------------------------------
//  RXgN^
procedure TForm1.FormCreate(Sender: TObject);
begin
    FBox.SetCaption(Self);
    FIniName := ChangeFileExt(ParamStr(0), '.ini');
    LoadSetting();
    FLock := false;
    FStdOut := TStringList.Create;
    FErrOut := TStringList.Create;
    Dropper := TShellDrop.Create(Self, DroppedAction);
    SetEnable();
end;

//------------------------------------------------------------------------------
//  fXgN^
procedure TForm1.FormDestroy(Sender: TObject);
begin
    Dropper.Free;
    FStdOut.Free;
    FErrOut.Free;
    SaveSetting();
end;

//------------------------------------------------------------------------------
//  I
procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
    CanClose := not FLock;
end;

//------------------------------------------------------------------------------
//  ƗpXgNA
procedure   TForm1.ClearOut();
begin
    FStdOut.Clear;
    FErrOut.Clear;
end;

//------------------------------------------------------------------------------
//  \
procedure   TForm1.ShowOut();
begin
    if FErrOut.Count > 0 then
        MBox.ErrorOk(FStdOut.Text, true)
    else
        MBox.InfoOk(FStdOut.Text, true);
end;

//------------------------------------------------------------------------------
//  t@Chbv
procedure   TForm1.DroppedAction(AFiles, AFolds: TStringList;
            AControl: TControl; APos: TPoint);
var
    i: integer;
begin
    if ((AControl <> GrpPack) and (AControl <> GrpUnPack)) then
    begin
        MBox.InfoOk('Ⴂ܂B', true);
        exit;
    end;

    Dropper.Stop();
    Screen.Cursor := crHourGlass;
    ClearOut();

    for i := 0 to AFiles.Count - 1 do
    begin
        if (AnsiPos(UpperCase(ExtractFileExt(AFiles[i])), '.EXE,.DLL') = 0) then continue;
        FLock := true;
        if (AControl = GrpPack) then
        begin
            if (TrkLevel.Position = 10) then DoBestCompress(AFiles[i])
                                        else DoLevelCompress(AFiles[i]);
        end
        else
        if (AControl = GrpUnPack) then
        begin
            DoDeCompress(AFiles[i]);
        end;
        FLock := false;

        //  ESC Œf
        if WinApi.IsKeyDown(VK_ESCAPE) then break;
    end;

    ShowOut();
    Screen.Cursor := crDefault;
    Dropper.Start();
end;


//------------------------------------------------------------------------------
//  Ǎ
procedure   TForm1.LoadSetting();
begin
    FIniFile := TIniFile.Create(FIniName);
    try
        Self.Left := FIniFile.ReadInteger(Self.Name, 'Left', Self.Left);
        Self.Top  := FIniFile.ReadInteger(Self.Name, 'Top', Self.Top);

        MenuPath.Caption := FIniFile.ReadString(MenuPath.Name, 'Caption', MenuPath.Caption);

        MenuBackup.Checked := FIniFile.ReadBool(MenuBackup.Name, 'Checked', MenuBackup.Checked);
        MenuExport.Checked := FIniFile.ReadBool(MenuExport.Name, 'Checked', MenuExport.Checked);
        MenuIcon.Checked   := FIniFile.ReadBool(MenuIcon.Name,   'Checked', MenuIcon.Checked);
    finally
        FIniFile.Free;
    end;
end;

//------------------------------------------------------------------------------
//  ۑ
procedure   TForm1.SaveSetting();
begin
    FIniFile := TIniFile.Create(FIniName);
    try
        FIniFile.WriteInteger(Self.Name, 'Left', Self.Left);
        FIniFile.WriteInteger(Self.Name, 'Top', Self.Top);

        FIniFile.WriteString(MenuPath.Name, 'Caption', MenuPath.Caption);

        FIniFile.WriteBool(MenuBackup.Name, 'Checked', MenuBackup.Checked);
        FIniFile.WriteBool(MenuExport.Name, 'Checked', MenuExport.Checked);
        FIniFile.WriteBool(MenuIcon.Name,   'Checked', MenuIcon.Checked);
    finally
        FIniFile.Free;
    end;
end;

//------------------------------------------------------------------------------
//  Upx̃pX
function    TForm1.UpxPath(): string;
begin
    Result := StripHotKey(MenuPath.Caption);
end;




//------------------------------------------------------------------------------
//  Upx̃pXĖ{H
procedure   TForm1.SetEnable();
var
    IsOK: Boolean;
begin
    IsOK := FileExists(ExpandFileName(UpxPath()));
    MenuSays.Enabled    := IsOK;
    MenuLicense.Enabled := IsOK;
    MenuVersion.Enabled := IsOK;
    if (IsOK) then Dropper.Start() else Dropper.Stop();
end;

//------------------------------------------------------------------------------
//  sƂʂƂ
procedure   TForm1.ExecuteWithShow(const Line: string);
begin
    ClearOut();
    PipeConsole(Line, FStdOut, FErrOut);
    ShowOut();
end;

//------------------------------------------------------------------------------
//  pXݒ
procedure TForm1.MenuPathClick(Sender: TObject);
var
    S, D: string;
begin
    if PShell.SelectDirectoryEx(Self, 'where is upx?', ExtractFilePath(ParamStr(0)), D) then
    begin
        S := '.\' + ExtractRelativePath(ParamStr(0), D + '\upx.exe');
        TMenuItem(Sender).Caption := S;
    end;
    SetEnable();
end;

//------------------------------------------------------------------------------
//  wv
procedure TForm1.MenuSaysClick(Sender: TObject);
begin
    ExecuteWithShow(Format('"%s"', [UpxPath()]));
end;

//------------------------------------------------------------------------------
//  LICENSE
procedure TForm1.MenuLicenseClick(Sender: TObject);
begin
    ExecuteWithShow(Format('"%s" -L', [UpxPath()]));
end;

//------------------------------------------------------------------------------
//  VERSION
procedure TForm1.MenuVersionClick(Sender: TObject);
begin
    ExecuteWithShow(Format('"%s" -V', [UpxPath()]));
end;























//------------------------------------------------------------------------------
//  kxύX
procedure TForm1.TrkLevelChange(Sender: TObject);
const
    LEVEL_STR: array[0..10] of string
    = ('none: ܂',
       '1   : ΂₭', '2   : ΂₭', '3   : ΂₭',
       '4   : Ăǂ', '5   : Ăǂ', '6   : Ăǂ',
       '7   : ', '8   : ', '9   : ',
       'best: '
    );
begin
    LblLevel.Caption := LEVEL_STR[TrkLevel.Position];
end;

//------------------------------------------------------------------------------
//  k
const
    BACK_CMD : array[Boolean] of string = ('', '-k');
    EXPO_CMD : array[Boolean] of string = ('', '--compress-exports=0') ;
    ICON_CMD : array[Boolean] of string = ('', '--compress-icons=0');

procedure   TForm1.DoLevelCompress(const Path: string);
const
    LEVEL_CMD: array[0..9] of string = ('', '-1', '-2', '-3', '-4', '-5', '-6', '-7', '-8', '-9');
    COMMAND_FMT: string = '"%s" --no-color %s %s %s %s "%s"';
var
    Level: integer;
    Line: string;
begin
    Level := TrkLevel.Position;
    //  xw舳k
    if (-1 < Level) and (Level < 10) then
    begin
        Line := Format(COMMAND_FMT, [
                            UpxPath(),                      //  UPXpX
                            LEVEL_CMD[Level],               //  k
                            BACK_CMD[MenuBackup.Checked],    //  obNAbv
                            EXPO_CMD[MenuExport.Checked],    //  GNX|[gk
                            ICON_CMD[MenuIcon.Checked],    //  ACRk
                            Path
        ]);
        PipeConsole(Line, FStdOut, FErrOut);
    end
end;

//------------------------------------------------------------------------------
//  xXgk
procedure   TForm1.DoBestCompress(const Path: string);
const
    BEST_CMD : array[0..1] of string = (
        '"%s" --best --crp-ms=999999 --nrv2b -o "%s" %s %s %s "%s"',
        '"%s" --best --crp-ms=999999 --nrv2d -o "%s" %s %s %s "%s"'
    );
var
    Dir, ResB, ResD: string;
    SizeB, SizeD: integer;
    Line, BackName: string;
begin
    Dir := ExtractFilePath(Path);
    ResB := WinApi.GetTempFile(Dir, '.exe');
    ResD := WinApi.GetTempFile(Dir, '.exe');
    DeleteFile(ResB);
    DeleteFile(ResD);

    Line := Format(BEST_CMD[0], [
                        UpxPath(),
                        ResB,
                            BACK_CMD[MenuBackup.Checked],    //  obNAbv
                            EXPO_CMD[MenuExport.Checked],    //  GNX|[gk
                            ICON_CMD[MenuIcon.Checked],    //  ACRk
                        Path
    ]);
    PipeConsole(Line, FStdOut, FErrOut);
    Line := Format(BEST_CMD[1], [
                        UpxPath(),
                        ResD,
                            BACK_CMD[MenuBackup.Checked],    //  obNAbv
                            EXPO_CMD[MenuExport.Checked],    //  GNX|[gk
                            ICON_CMD[MenuIcon.Checked],    //  ACRk
                        Path
    ]);
    PipeConsole(Line, FStdOut, FErrOut);

    if (FErrOut.Count > 0) then exit;

    SizeB := WinApi.FileSize(ResB);
    SizeD := WinApi.FileSize(ResD);

    BackName := ChangeFileExt(Path, '.' + FormatDateTime('yyyymmddhhnnss', Now()));
    RenameFile(Path, BackName);

    if (SizeB <= SizeD) then
    begin
        RenameFile(ResB, Path);
        DeleteFile(ResD);
    end
    else
    begin
        RenameFile(ResD, Path);
        DeleteFile(ResB);
    end;
end;

//------------------------------------------------------------------------------
//  
procedure   TForm1.DoDeCompress(const Path: string);
var
    Line: string;
begin
    Line := Format('"%s"-d --no-color "%s"', [UpxPath(), Path]);
    PipeConsole(Line, FStdOut, FErrOut);
end;







end.

